@@ -1,10 +1,11 @@
 # UFL input for the Matrix-free Poisson Demo
 # ==================================
+from basix.ufl_wrapper import create_vector_element
 from ufl import (Coefficient, Constant, FiniteElement, FunctionSpace, Mesh,
-                 TestFunction, TrialFunction, VectorElement, action, dx, grad,
-                 inner, triangle)
+                 TestFunction, TrialFunction, action, dx, grad, inner,
+                 triangle)
 
-coord_element = VectorElement("Lagrange", triangle, 1)
+coord_element = create_vector_element("Lagrange", "triangle", 1)
 mesh = Mesh(coord_element)
 
 # Function Space
