@@ -1,5 +1,5 @@
-// Copyright (C) 2006-2021 Chris N. Richardson, Anders Logg, Garth N. Wells and
-// Jørgen S. Dokken
+// Copyright (C) 2006-2022 Chris N. Richardson, Anders Logg, Garth N. Wells,
+// Jørgen S. Dokken, Sarah Roggendorf
 //
 // This file is part of DOLFINx (https://www.fenicsproject.org)
 //
@@ -9,7 +9,9 @@
 #include "BoundingBoxTree.h"
 #include "gjk.h"
 #include <deque>
+#include <dolfinx/common/IndexMap.h>
 #include <dolfinx/common/log.h>
+#include <dolfinx/common/utils.h>
 #include <dolfinx/mesh/Geometry.h>
 #include <dolfinx/mesh/Mesh.h>
 #include <dolfinx/mesh/utils.h>
@@ -19,64 +21,76 @@ using namespace dolfinx;
 namespace
 {
 //-----------------------------------------------------------------------------
-// Check whether bounding box is a leaf node
-constexpr bool is_leaf(const std::array<int, 2>& bbox)
+
+/// Check whether bounding box is a leaf node
+constexpr bool is_leaf(std::array<int, 2> bbox)
 {
   // Leaf nodes are marked by setting child_0 equal to child_1
   return bbox[0] == bbox[1];
 }
 //-----------------------------------------------------------------------------
+
 /// A point `x` is inside a bounding box `b` if each component of its
-/// coordinates lies within the range `[b(0,i), b(1,i)]` that defines the bounds
-/// of the bounding box, b(0,i) <= x[i] <= b(1,i) for i = 0, 1, 2
-constexpr bool point_in_bbox(const std::array<std::array<double, 3>, 2>& b,
-                             const std::array<double, 3>& x)
+/// coordinates lies within the range `[b(0,i), b(1,i)]` that defines
+/// the bounds of the bounding box, b(0,i) <= x[i] <= b(1,i) for i = 0,
+/// 1, 2
+constexpr bool point_in_bbox(const std::array<double, 6>& b,
+                             std::span<const double, 3> x)
 {
+  assert(b.size() == 6);
   constexpr double rtol = 1e-14;
   bool in = true;
-  for (int i = 0; i < 3; i++)
+  for (std::size_t i = 0; i < 3; i++)
   {
-    double eps = rtol * (b[1][i] - b[0][i]);
-    in &= x[i] >= (b[0][i] - eps);
-    in &= x[i] <= (b[1][i] + eps);
+    double eps = rtol * (b[i + 3] - b[i]);
+    in &= x[i] >= (b[i] - eps);
+    in &= x[i] <= (b[i + 3] + eps);
   }
 
   return in;
 }
 //-----------------------------------------------------------------------------
-/// A bounding box "a" is contained inside another bounding box "b", if each
-/// of its intervals [a(0,i), a(1,i)] is contained in [b(0,i), b(1,i)],
-/// a(0,i) <= b(1, i) and a(1,i) >= b(0, i)
-constexpr bool bbox_in_bbox(const std::array<std::array<double, 3>, 2>& a,
-                            const std::array<std::array<double, 3>, 2>& b)
+
+/// A bounding box "a" is contained inside another bounding box "b", if
+/// each  of its intervals [a(0,i), a(1,i)] is contained in [b(0,i),
+/// b(1,i)], a(0,i) <= b(1, i) and a(1,i) >= b(0, i)
+constexpr bool bbox_in_bbox(std::span<const double, 6> a,
+                            std::span<const double, 6> b)
 {
   constexpr double rtol = 1e-14;
+  auto a0 = a.subspan<0, 3>();
+  auto a1 = a.subspan<3, 3>();
+  auto b0 = b.subspan<0, 3>();
+  auto b1 = b.subspan<3, 3>();
+
   bool in = true;
-  for (int i = 0; i < 3; i++)
+  for (std::size_t i = 0; i < 3; i++)
   {
-    double eps = rtol * (b[1][i] - b[0][i]);
-    in &= a[1][i] >= (b[0][i] - eps);
-    in &= a[0][i] <= (b[1][i] + eps);
+    double eps = rtol * (b1[i] - b0[i]);
+    in &= a1[i] >= (b0[i] - eps);
+    in &= a0[i] <= (b1[i] + eps);
   }
 
   return in;
 }
 //-----------------------------------------------------------------------------
 // Compute closest entity {closest_entity, R2} (recursive)
-std::pair<std::int32_t, double> _compute_closest_entity(
-    const geometry::BoundingBoxTree& tree, const std::array<double, 3>& point,
-    int node, const mesh::Mesh& mesh, std::int32_t closest_entity, double R2)
+std::pair<std::int32_t, double>
+_compute_closest_entity(const geometry::BoundingBoxTree& tree,
+                        std::span<const double, 3> point, std::int32_t node,
+                        const mesh::Mesh& mesh, std::int32_t closest_entity,
+                        double R2)
 {
   // Get children of current bounding box node (child_1 denotes entity
   // index for leaves)
-  const std::array bbox = tree.bbox(node);
+  const std::array<int, 2> bbox = tree.bbox(node);
   double r2;
   if (is_leaf(bbox))
   {
     // If point cloud tree the exact distance is easy to compute
     if (tree.tdim() == 0)
     {
-      std::array<double, 3> diff = tree.get_bbox(node)[0];
+      std::array<double, 6> diff = tree.get_bbox(node);
       for (std::size_t k = 0; k < 3; ++k)
         diff[k] -= point[k];
       r2 = diff[0] * diff[0] + diff[1] * diff[1] + diff[2] * diff[2];
@@ -84,13 +98,14 @@ std::pair<std::int32_t, double> _compute_closest_entity(
     else
     {
       r2 = geometry::compute_squared_distance_bbox(tree.get_bbox(node), point);
+
       // If bounding box closer than previous closest entity, use gjk to
       // obtain exact distance to the convex hull of the entity
       if (r2 <= R2)
       {
-        r2 = geometry::squared_distance(mesh, tree.tdim(),
-                                        std::span(&bbox[1], 1),
-                                        {{point[0], point[1], point[2]}})
+        r2 = geometry::squared_distance(
+                 mesh, tree.tdim(), std::span(std::next(bbox.begin(), 1), 1),
+                 point)
                  .front();
       }
     }
@@ -98,7 +113,7 @@ std::pair<std::int32_t, double> _compute_closest_entity(
     // If entity is closer than best result so far, return it
     if (r2 <= R2)
     {
-      closest_entity = bbox[1];
+      closest_entity = bbox.back();
       R2 = r2;
     }
 
@@ -115,9 +130,9 @@ std::pair<std::int32_t, double> _compute_closest_entity(
     // We use R2 (as opposed to r2), as a bounding box can be closer
     // than the actual entity
     std::pair<int, double> p0 = _compute_closest_entity(
-        tree, point, bbox[0], mesh, closest_entity, R2);
+        tree, point, bbox.front(), mesh, closest_entity, R2);
     std::pair<int, double> p1 = _compute_closest_entity(
-        tree, point, bbox[1], mesh, p0.first, p0.second);
+        tree, point, bbox.back(), mesh, p0.first, p0.second);
     return p1;
   }
 }
@@ -127,15 +142,14 @@ std::pair<std::int32_t, double> _compute_closest_entity(
 /// @param[in] points The points (shape=(num_points, 3))
 /// @param[in, out] entities The list of colliding entities (local to process)
 void _compute_collisions_point(const geometry::BoundingBoxTree& tree,
-                               const std::array<double, 3>& p,
-                               std::vector<int>& entities)
+                               std::span<const double, 3> p,
+                               std::vector<std::int32_t>& entities)
 {
   std::deque<std::int32_t> stack;
-  int next = tree.num_bboxes() - 1;
-
+  std::int32_t next = tree.num_bboxes() - 1;
   while (next != -1)
   {
-    std::array bbox = tree.bbox(next);
+    const std::array<int, 2> bbox = tree.bbox(next);
     next = -1;
 
     if (is_leaf(bbox))
@@ -179,17 +193,17 @@ void _compute_collisions_point(const geometry::BoundingBoxTree& tree,
 //-----------------------------------------------------------------------------
 // Compute collisions with tree (recursive)
 void _compute_collisions_tree(const geometry::BoundingBoxTree& A,
-                              const geometry::BoundingBoxTree& B, int node_A,
-                              int node_B,
-                              std::vector<std::array<int, 2>>& entities)
+                              const geometry::BoundingBoxTree& B,
+                              std::int32_t node_A, std::int32_t node_B,
+                              std::vector<std::int32_t>& entities)
 {
   // If bounding boxes don't collide, then don't search further
   if (!bbox_in_bbox(A.get_bbox(node_A), B.get_bbox(node_B)))
     return;
 
   // Get bounding boxes for current nodes
-  const std::array bbox_A = A.bbox(node_A);
-  const std::array bbox_B = B.bbox(node_B);
+  const std::array<std::int32_t, 2> bbox_A = A.bbox(node_A);
+  const std::array<std::int32_t, 2> bbox_B = B.bbox(node_B);
 
   // Check whether we've reached a leaf in A or B
   const bool is_leaf_A = is_leaf(bbox_A);
@@ -198,7 +212,8 @@ void _compute_collisions_tree(const geometry::BoundingBoxTree& A,
   {
     // If both boxes are leaves (which we know collide), then add them
     // child_1 denotes entity for leaves
-    entities.push_back({bbox_A[1], bbox_B[1]});
+    entities.push_back(bbox_A[1]);
+    entities.push_back(bbox_B[1]);
   }
   else if (is_leaf_A)
   {
@@ -237,7 +252,7 @@ void _compute_collisions_tree(const geometry::BoundingBoxTree& A,
 //-----------------------------------------------------------------------------
 geometry::BoundingBoxTree
 geometry::create_midpoint_tree(const mesh::Mesh& mesh, int tdim,
-                               const std::span<const std::int32_t>& entities)
+                               std::span<const std::int32_t> entities)
 {
   LOG(INFO) << "Building point search tree to accelerate distance queries for "
                "a given topological dimension and subset of entities.";
@@ -257,12 +272,12 @@ geometry::create_midpoint_tree(const mesh::Mesh& mesh, int tdim,
   return geometry::BoundingBoxTree(points);
 }
 //-----------------------------------------------------------------------------
-std::vector<std::array<int, 2>>
+std::vector<std::int32_t>
 geometry::compute_collisions(const BoundingBoxTree& tree0,
                              const BoundingBoxTree& tree1)
 {
   // Call recursive find function
-  std::vector<std::array<int, 2>> entities;
+  std::vector<std::int32_t> entities;
   if (tree0.num_bboxes() > 0 and tree1.num_bboxes() > 0)
   {
     _compute_collisions_tree(tree0, tree1, tree0.num_bboxes() - 1,
@@ -274,7 +289,7 @@ geometry::compute_collisions(const BoundingBoxTree& tree0,
 //-----------------------------------------------------------------------------
 graph::AdjacencyList<std::int32_t>
 geometry::compute_collisions(const BoundingBoxTree& tree,
-                             const std::span<const double>& points)
+                             std::span<const double> points)
 {
   if (tree.num_bboxes() > 0)
   {
@@ -283,8 +298,7 @@ geometry::compute_collisions(const BoundingBoxTree& tree,
     for (std::size_t p = 0; p < points.size() / 3; ++p)
     {
       _compute_collisions_point(
-          tree, {points[3 * p + 0], points[3 * p + 1], points[3 * p + 2]},
-          entities);
+          tree, std::span<const double, 3>(points.data() + 3 * p, 3), entities);
       offsets[p + 1] = entities.size();
     }
 
@@ -301,7 +315,7 @@ geometry::compute_collisions(const BoundingBoxTree& tree,
 //-----------------------------------------------------------------------------
 std::vector<std::int32_t> geometry::compute_closest_entity(
     const BoundingBoxTree& tree, const BoundingBoxTree& midpoint_tree,
-    const mesh::Mesh& mesh, const std::span<const double>& points)
+    const mesh::Mesh& mesh, std::span<const double> points)
 {
   if (tree.num_bboxes() == 0)
     return std::vector<std::int32_t>(points.size() / 3, -1);
@@ -320,7 +334,7 @@ std::vector<std::int32_t> geometry::compute_closest_entity(
       leaves = midpoint_tree.bbox(0);
       assert(is_leaf(leaves));
       initial_entity = leaves[0];
-      std::array<double, 3> diff = midpoint_tree.get_bbox(0)[0];
+      std::array<double, 6> diff = midpoint_tree.get_bbox(0);
       for (std::size_t k = 0; k < 3; ++k)
         diff[k] -= points[3 * i + k];
       R2 = diff[0] * diff[0] + diff[1] * diff[1] + diff[2] * diff[2];
@@ -330,8 +344,7 @@ std::vector<std::int32_t> geometry::compute_closest_entity(
       // As the midpoint tree only consist of points, the distance
       // queries are lightweight.
       const auto [m_index, m_distance2] = _compute_closest_entity(
-          midpoint_tree,
-          {points[3 * i + 0], points[3 * i + 1], points[3 * i + 2]},
+          midpoint_tree, std::span<const double, 3>(points.data() + 3 * i, 3),
           midpoint_tree.num_bboxes() - 1, mesh, initial_entity, R2);
 
       // Use a recursive search through the bounding box tree to
@@ -340,7 +353,7 @@ std::vector<std::int32_t> geometry::compute_closest_entity(
       // the distance from the midpoint to the point of interest as the
       // initial search radius.
       const auto [index, distance2] = _compute_closest_entity(
-          tree, {points[3 * i + 0], points[3 * i + 1], points[3 * i + 2]},
+          tree, std::span<const double, 3>(points.data() + 3 * i, 3),
           tree.num_bboxes() - 1, mesh, m_index, m_distance2);
 
       entities.push_back(index);
@@ -351,12 +364,12 @@ std::vector<std::int32_t> geometry::compute_closest_entity(
 }
 
 //-----------------------------------------------------------------------------
-double geometry::compute_squared_distance_bbox(
-    const std::array<std::array<double, 3>, 2>& b,
-    const std::array<double, 3>& x)
+double geometry::compute_squared_distance_bbox(std::span<const double, 6> b,
+                                               std::span<const double, 3> x)
 {
-  auto& b0 = b[0];
-  auto& b1 = b[1];
+  assert(b.size() == 6);
+  auto b0 = b.subspan<0, 3>();
+  auto b1 = b.subspan<3, 3>();
   return std::transform_reduce(x.begin(), x.end(), b0.begin(), 0.0,
                                std::plus<>{},
                                [](auto x, auto b)
@@ -375,8 +388,8 @@ double geometry::compute_squared_distance_bbox(
 //-----------------------------------------------------------------------------
 std::vector<double>
 geometry::shortest_vector(const mesh::Mesh& mesh, int dim,
-                          const std::span<const std::int32_t>& entities,
-                          const std::span<const double>& points)
+                          std::span<const std::int32_t> entities,
+                          std::span<const double> points)
 {
   const int tdim = mesh.topology().dim();
   const mesh::Geometry& geometry = mesh.geometry();
@@ -447,8 +460,8 @@ geometry::shortest_vector(const mesh::Mesh& mesh, int dim,
 //-----------------------------------------------------------------------------
 std::vector<double>
 geometry::squared_distance(const mesh::Mesh& mesh, int dim,
-                           const std::span<const std::int32_t>& entities,
-                           const std::span<const double>& points)
+                           std::span<const std::int32_t> entities,
+                           std::span<const double> points)
 {
   std::vector<double> v = shortest_vector(mesh, dim, entities, points);
   std::vector<double> d(v.size() / 3, 0);
@@ -461,7 +474,7 @@ geometry::squared_distance(const mesh::Mesh& mesh, int dim,
 graph::AdjacencyList<std::int32_t> geometry::compute_colliding_cells(
     const mesh::Mesh& mesh,
     const graph::AdjacencyList<std::int32_t>& candidate_cells,
-    const std::span<const double>& points)
+    std::span<const double> points)
 {
   std::vector<std::int32_t> offsets = {0};
   offsets.reserve(candidate_cells.num_nodes() + 1);
@@ -489,3 +502,247 @@ graph::AdjacencyList<std::int32_t> geometry::compute_colliding_cells(
                                             std::move(offsets));
 }
 //-------------------------------------------------------------------------------
+int geometry::compute_first_colliding_cell(
+    const mesh::Mesh& mesh, const geometry::BoundingBoxTree& tree,
+    const std::array<double, 3>& point)
+{
+  // Compute colliding bounding boxes(cell candidates)
+  std::vector<std::int32_t> cell_candidates;
+  _compute_collisions_point(tree, point, cell_candidates);
+
+  if (cell_candidates.empty())
+    return -1;
+  else
+  {
+    constexpr double eps2 = 1e-20;
+    const mesh::Geometry& geometry = mesh.geometry();
+    std::span<const double> geom_dofs = geometry.x();
+    const graph::AdjacencyList<std::int32_t>& x_dofmap = geometry.dofmap();
+    const std::size_t num_nodes = geometry.cmap().dim();
+    std::vector<double> coordinate_dofs(num_nodes * 3);
+    for (auto cell : cell_candidates)
+    {
+      auto dofs = x_dofmap.links(cell);
+      for (std::size_t i = 0; i < num_nodes; ++i)
+      {
+        std::copy(std::next(geom_dofs.begin(), 3 * dofs[i]),
+                  std::next(geom_dofs.begin(), 3 * dofs[i] + 3),
+                  std::next(coordinate_dofs.begin(), 3 * i));
+      }
+      std::array<double, 3> shortest_vector
+          = geometry::compute_distance_gjk(point, coordinate_dofs);
+      double norm = 0;
+      std::for_each(shortest_vector.cbegin(), shortest_vector.cend(),
+                    [&norm](const double e) { norm += std::pow(e, 2); });
+
+      if (norm < eps2)
+        return cell;
+    }
+  }
+  return -1;
+}
+
+//-------------------------------------------------------------------------------
+std::tuple<std::vector<std::int32_t>, std::vector<std::int32_t>,
+           std::vector<double>, std::vector<std::int32_t>>
+geometry::determine_point_ownership(const mesh::Mesh& mesh,
+                                    std::span<const double> points)
+{
+  const MPI_Comm& comm = mesh.comm();
+
+  // Create a global bounding-box tree to find candidate processes with cells
+  // that could collide with thte points
+  constexpr double padding = 0.0001;
+  const int tdim = mesh.topology().dim();
+  const auto cell_map = mesh.topology().index_map(tdim);
+  const int num_cells = cell_map->size_local();
+  // NOTE: Should we send the cells in as input?
+  std::vector<std::int32_t> cells(num_cells, 0);
+  std::iota(cells.begin(), cells.end(), 0);
+  BoundingBoxTree bb(mesh, tdim, cells, padding);
+  BoundingBoxTree global_bbtree = bb.create_global_tree(comm);
+
+  // Compute collisions:
+  // For each point in `x` get the processes it should be sent to
+  graph::AdjacencyList<std::int32_t> collisions
+      = compute_collisions(global_bbtree, points);
+
+  // Get unique list of outgoing ranks
+  std::vector<std::int32_t> out_ranks = collisions.array();
+  std::sort(out_ranks.begin(), out_ranks.end());
+  out_ranks.erase(std::unique(out_ranks.begin(), out_ranks.end()),
+                  out_ranks.end());
+
+  // Compute incoming edges (source processes)
+  std::vector<int> in_ranks
+      = dolfinx::MPI::compute_graph_edges_nbx(comm, out_ranks);
+  std::sort(in_ranks.begin(), in_ranks.end());
+
+  // Create neighborhood communicator in forward direction
+  MPI_Comm forward_comm;
+  MPI_Dist_graph_create_adjacent(
+      comm, in_ranks.size(), in_ranks.data(), MPI_UNWEIGHTED, out_ranks.size(),
+      out_ranks.data(), MPI_UNWEIGHTED, MPI_INFO_NULL, false, &forward_comm);
+
+  // Compute map from global mpi rank to neighbor rank, "collisions" uses
+  // global rank
+  std::map<std::int32_t, std::int32_t> rank_to_neighbor;
+  for (std::size_t i = 0; i < out_ranks.size(); i++)
+    rank_to_neighbor[out_ranks[i]] = i;
+
+  // Count the number of points to send per neighbor process
+  std::vector<std::int32_t> send_sizes(out_ranks.size());
+  for (std::size_t i = 0; i < points.size() / 3; ++i)
+    for (const auto& p : collisions.links(i))
+      send_sizes[rank_to_neighbor[p]] += 3;
+
+  // Compute receive sizes
+  std::vector<std::int32_t> recv_sizes(in_ranks.size());
+  send_sizes.reserve(1);
+  recv_sizes.reserve(1);
+  MPI_Request sizes_request;
+  MPI_Ineighbor_alltoall(send_sizes.data(), 1, MPI_INT, recv_sizes.data(), 1,
+                         MPI_INT, forward_comm, &sizes_request);
+
+  // Compute sending offsets
+  std::vector<std::int32_t> send_offsets(send_sizes.size() + 1, 0);
+  std::partial_sum(send_sizes.begin(), send_sizes.end(),
+                   std::next(send_offsets.begin(), 1));
+
+  // Pack data to send and store unpack map
+  std::vector<double> send_data(send_offsets.back());
+  std::vector<std::int32_t> counter(send_sizes.size(), 0);
+  // unpack map: [index in adj list][pos in x]
+  std::vector<std::int32_t> unpack_map(send_offsets.back() / 3);
+  for (std::size_t i = 0; i < points.size(); i += 3)
+  {
+    for (const auto& p : collisions.links(i / 3))
+    {
+      int neighbor = rank_to_neighbor[p];
+      int pos = send_offsets[neighbor] + counter[neighbor];
+      auto it = std::next(send_data.begin(), pos);
+      std::copy(std::next(points.begin(), i), std::next(points.begin(), i + 3),
+                it);
+      unpack_map[pos / 3] = i / 3;
+      counter[neighbor] += 3;
+    }
+  }
+
+  MPI_Wait(&sizes_request, MPI_STATUS_IGNORE);
+  std::vector<std::int32_t> recv_offsets(in_ranks.size() + 1, 0);
+  std::partial_sum(recv_sizes.begin(), recv_sizes.end(),
+                   std::next(recv_offsets.begin(), 1));
+
+  std::vector<double> received_points((std::size_t)recv_offsets.back());
+  MPI_Neighbor_alltoallv(send_data.data(), send_sizes.data(),
+                         send_offsets.data(), MPI_DOUBLE,
+                         received_points.data(), recv_sizes.data(),
+                         recv_offsets.data(), MPI_DOUBLE, forward_comm);
+
+  // Each process checks which points collides with a cell on the process
+  const int rank = dolfinx::MPI::rank(comm);
+  std::vector<std::int32_t> cell_indicator(received_points.size() / 3);
+  std::vector<std::int32_t> colliding_cells(received_points.size() / 3);
+  for (std::size_t p = 0; p < received_points.size(); p += 3)
+  {
+    // NOTE: Aim to remove this by using span, see:
+    // https://github.com/FEniCS/dolfinx/issues/2284
+    std::array<double, 3> point;
+    std::copy(std::next(received_points.begin(), p),
+              std::next(received_points.begin(), p + 3), point.begin());
+    const int colliding_cell
+        = geometry::compute_first_colliding_cell(mesh, bb, point);
+    cell_indicator[p / 3] = (colliding_cell >= 0) ? rank : -1;
+    colliding_cells[p / 3] = colliding_cell;
+  }
+  // Create neighborhood communicator in the reverse direction: send back col to
+  // requesting processes
+  MPI_Comm reverse_comm;
+  MPI_Dist_graph_create_adjacent(
+      comm, out_ranks.size(), out_ranks.data(), MPI_UNWEIGHTED, in_ranks.size(),
+      in_ranks.data(), MPI_UNWEIGHTED, MPI_INFO_NULL, false, &reverse_comm);
+
+  // Reuse sizes and offsets from first communication set
+  // but divide by three
+  {
+    auto rescale = [](auto& x)
+    {
+      std::transform(x.cbegin(), x.cend(), x.begin(),
+                     [](auto e) { return (e / 3); });
+    };
+    rescale(recv_sizes);
+    rescale(recv_offsets);
+    rescale(send_sizes);
+    rescale(send_offsets);
+
+    // The communication is reversed, so swap recv to send offsets
+    std::swap(recv_sizes, send_sizes);
+    std::swap(recv_offsets, send_offsets);
+  }
+
+  std::vector<std::int32_t> recv_ranks(recv_offsets.back());
+  MPI_Neighbor_alltoallv(
+      cell_indicator.data(), send_sizes.data(), send_offsets.data(),
+      dolfinx::MPI::mpi_type<std::int32_t>(), recv_ranks.data(),
+      recv_sizes.data(), recv_offsets.data(),
+      dolfinx::MPI::mpi_type<std::int32_t>(), reverse_comm);
+
+  std::vector<std::int32_t> point_owners(points.size() / 3, -1);
+  for (std::size_t i = 0; i < unpack_map.size(); i++)
+  {
+    const std::int32_t pos = unpack_map[i];
+    // Only insert new owner if no owner has previously been found
+    if ((recv_ranks[i] >= 0) && (point_owners[pos] == -1))
+      point_owners[pos] = recv_ranks[i];
+  }
+  // Communication is reversed again to send dest ranks to all processes
+  std::swap(send_sizes, recv_sizes);
+  std::swap(send_offsets, recv_offsets);
+
+  // Pack ownership data
+  std::vector<std::int32_t> send_owners(send_offsets.back());
+  std::fill(counter.begin(), counter.end(), 0);
+  for (std::size_t i = 0; i < points.size() / 3; ++i)
+  {
+    for (const auto& p : collisions.links(i))
+    {
+      int neighbor = rank_to_neighbor[p];
+      send_owners[send_offsets[neighbor] + counter[neighbor]++]
+          = point_owners[i];
+    }
+  }
+
+  // Send ownership info
+  std::vector<std::int32_t> dest_ranks(recv_offsets.back());
+  MPI_Neighbor_alltoallv(
+      send_owners.data(), send_sizes.data(), send_offsets.data(),
+      dolfinx::MPI::mpi_type<std::int32_t>(), dest_ranks.data(),
+      recv_sizes.data(), recv_offsets.data(),
+      dolfinx::MPI::mpi_type<std::int32_t>(), forward_comm);
+
+  // Unpack dest ranks if point owner is this rank
+  std::vector<std::int32_t> owned_recv_ranks;
+  owned_recv_ranks.reserve(recv_offsets.back());
+  std::vector<double> owned_recv_points;
+  std::vector<std::int32_t> owned_recv_cells;
+  for (std::size_t i = 0; i < in_ranks.size(); i++)
+  {
+    for (std::int32_t j = recv_offsets[i]; j < recv_offsets[i + 1]; j++)
+    {
+      if (rank == dest_ranks[j])
+      {
+        owned_recv_ranks.push_back(in_ranks[i]);
+        owned_recv_points.insert(
+            owned_recv_points.end(), std::next(received_points.cbegin(), 3 * j),
+            std::next(received_points.cbegin(), 3 * (j + 1)));
+        owned_recv_cells.push_back(colliding_cells[j]);
+      }
+    }
+  }
+
+  MPI_Comm_free(&forward_comm);
+  MPI_Comm_free(&reverse_comm);
+
+  return std::make_tuple(point_owners, owned_recv_ranks, owned_recv_points,
+                         owned_recv_cells);
+};
\ No newline at end of file
