@@ -12,8 +12,8 @@
 #include "graphbuild.h"
 #include "topologycomputation.h"
 #include "utils.h"
+#include <algorithm>
 #include <dolfinx/common/IndexMap.h>
-#include <dolfinx/common/utils.h>
 #include <dolfinx/fem/CoordinateElement.h>
 #include <dolfinx/graph/AdjacencyList.h>
 #include <dolfinx/graph/ordering.h>
@@ -27,9 +27,8 @@ namespace
 {
 /// Re-order an adjacency list
 template <typename T>
-graph::AdjacencyList<T>
-reorder_list(const graph::AdjacencyList<T>& list,
-             const std::span<const std::int32_t>& nodemap)
+graph::AdjacencyList<T> reorder_list(const graph::AdjacencyList<T>& list,
+                                     std::span<const std::int32_t> nodemap)
 {
   // Copy existing data to keep ghost values (not reordered)
   std::vector<T> data(list.array());
@@ -64,8 +63,8 @@ Mesh mesh::create_mesh(MPI_Comm comm,
                        std::array<std::size_t, 2> xshape,
                        mesh::GhostMode ghost_mode)
 {
-  return create_mesh(comm, cells, element, x, xshape, ghost_mode,
-                     create_cell_partitioner());
+  return create_mesh(comm, cells, element, x, xshape,
+                     create_cell_partitioner(ghost_mode));
 }
 //-----------------------------------------------------------------------------
 Mesh mesh::create_mesh(MPI_Comm comm,
@@ -73,17 +72,13 @@ Mesh mesh::create_mesh(MPI_Comm comm,
                        const fem::CoordinateElement& element,
                        std::span<const double> x,
                        std::array<std::size_t, 2> xshape,
-                       mesh::GhostMode ghost_mode,
                        const mesh::CellPartitionFunction& cell_partitioner)
 {
-  if (ghost_mode == GhostMode::shared_vertex)
-    throw std::runtime_error("Ghost mode via vertex currently disabled.");
-
   const fem::ElementDofLayout dof_layout = element.create_dof_layout();
 
   // Function top build geometry. Used to scope memory operations.
   auto build_topology = [](auto comm, auto& element, auto& dof_layout,
-                           auto& cells, auto ghost_mode, auto& cell_partitioner)
+                           auto& cells, auto& cell_partitioner)
   {
     // -- Partition topology
 
@@ -99,14 +94,12 @@ Mesh mesh::create_mesh(MPI_Comm comm,
     // it is just the identity
 
     // Compute the destination rank for cells on this process via graph
-    // partitioning. Always get the ghost cells via facet, though these
-    // may be discarded later.
+    // partitioning.
     const int size = dolfinx::MPI::size(comm);
     const int tdim = cell_dim(element.cell_shape());
     const graph::AdjacencyList<std::int32_t> dest = cell_partitioner(
         comm, size, tdim,
-        extract_topology(element.cell_shape(), dof_layout, cells),
-        GhostMode::shared_facet);
+        extract_topology(element.cell_shape(), dof_layout, cells));
 
     // -- Distribute cells (topology, includes higher-order 'nodes')
 
@@ -121,6 +114,7 @@ Mesh mesh::create_mesh(MPI_Comm comm,
 
     // Extract cell 'topology', i.e. extract the vertices for each cell
     // and discard any 'higher-order' nodes
+
     graph::AdjacencyList<std::int64_t> cells_extracted
         = extract_topology(element.cell_shape(), dof_layout, cell_nodes);
 
@@ -129,14 +123,17 @@ Mesh mesh::create_mesh(MPI_Comm comm,
     // Build local dual graph for owned cells to apply re-ordering to
     const std::int32_t num_owned_cells
         = cells_extracted.num_nodes() - ghost_owners.size();
-    const std::vector<int> remap
-        = graph::reorder_gps(std::get<0>(build_local_dual_graph(
+
+    auto [graph, unmatched_facets, max_v, facet_attached_cells]
+        = build_local_dual_graph(
             std::span<const std::int64_t>(
                 cells_extracted.array().data(),
                 cells_extracted.offsets()[num_owned_cells]),
             std::span<const std::int32_t>(cells_extracted.offsets().data(),
                                           num_owned_cells + 1),
-            tdim)));
+            tdim);
+
+    const std::vector<int> remap = graph::reorder_gps(graph);
 
     // Create re-ordered cell lists (leaves ghosts unchanged)
     std::vector<std::int64_t> original_cell_index(original_cell_index0.size());
@@ -150,17 +147,28 @@ Mesh mesh::create_mesh(MPI_Comm comm,
 
     // -- Create Topology
 
+    // Boundary vertices are marked as unknown
+    std::vector<std::int64_t> boundary_vertices(unmatched_facets);
+    std::sort(boundary_vertices.begin(), boundary_vertices.end());
+    boundary_vertices.erase(
+        std::unique(boundary_vertices.begin(), boundary_vertices.end()),
+        boundary_vertices.end());
+
+    // Remove -1 if it occurs in boundary vertices (may occur in mixed topology)
+    if (boundary_vertices.size() > 0 and boundary_vertices[0] == -1)
+      boundary_vertices.erase(boundary_vertices.begin());
+
     // Create cells and vertices with the ghosting requested. Input
     // topology includes cells shared via facet, but ghosts will be
     // removed later if not required by ghost_mode.
     return std::pair{create_topology(comm, cells_extracted, original_cell_index,
                                      ghost_owners, element.cell_shape(),
-                                     ghost_mode),
+                                     boundary_vertices),
                      std::move(cell_nodes)};
   };
 
-  auto [topology, cell_nodes] = build_topology(comm, element, dof_layout, cells,
-                                               ghost_mode, cell_partitioner);
+  auto [topology, cell_nodes]
+      = build_topology(comm, element, dof_layout, cells, cell_partitioner);
 
   // Create connectivity required to compute the Geometry (extra
   // connectivities for higher-order geometries)
@@ -168,154 +176,130 @@ Mesh mesh::create_mesh(MPI_Comm comm,
   for (int e = 1; e < tdim; ++e)
   {
     if (dof_layout.num_entity_dofs(e) > 0)
-    {
-      auto [cell_entity, entity_vertex, index_map]
-          = compute_entities(comm, topology, e);
-      if (cell_entity)
-        topology.set_connectivity(cell_entity, tdim, e);
-      if (entity_vertex)
-        topology.set_connectivity(entity_vertex, e, 0);
-      if (index_map)
-        topology.set_index_map(e, index_map);
-    }
+      topology.create_entities(e);
   }
 
-  // Function top build geometry. Used to scope memory operations.
-  auto build_geometry = [](auto comm, auto& cell_nodes, auto& topology,
-                           auto& element, auto& x, auto xshape1)
-  {
-    int tdim = topology.dim();
-    int num_cells = topology.index_map(tdim)->size_local()
-                    + topology.index_map(tdim)->num_ghosts();
-
-    // Remove ghost cells from geometry data, if not required
-    cell_nodes.offsets().resize(num_cells + 1);
-    cell_nodes.array().resize(cell_nodes.offsets().back());
-
-    if (element.needs_dof_permutations())
-      topology.create_entity_permutations();
-
-    return create_geometry(comm, topology, element, cell_nodes, x, xshape1);
-  };
+  if (element.needs_dof_permutations())
+    topology.create_entity_permutations();
 
   Geometry geometry
-      = build_geometry(comm, cell_nodes, topology, element, x, xshape[1]);
+      = create_geometry(comm, topology, element, cell_nodes, x, xshape[1]);
   return Mesh(comm, std::move(topology), std::move(geometry));
 }
 //-----------------------------------------------------------------------------
 std::tuple<Mesh, std::vector<std::int32_t>, std::vector<std::int32_t>,
            std::vector<std::int32_t>>
 mesh::create_submesh(const Mesh& mesh, int dim,
-                     const std::span<const std::int32_t>& entities)
+                     std::span<const std::int32_t> entities)
 {
   // -- Submesh topology
-
-  // Get the verticies in the submesh
-  std::vector<std::int32_t> submesh_vertices
-      = compute_incident_entities(mesh, entities, dim, 0);
-
-  // Get the vertices in the submesh owned by this process
-  auto mesh_vertex_index_map = mesh.topology().index_map(0);
-  assert(mesh_vertex_index_map);
-  std::vector<int32_t> submesh_owned_vertices
-      = dolfinx::common::compute_owned_indices(submesh_vertices,
-                                               *mesh_vertex_index_map);
-
-  // Create submesh vertex index map
-  std::pair<common::IndexMap, std::vector<int32_t>>
-      submesh_vertex_index_map_pair
-      = mesh_vertex_index_map->create_submap(submesh_owned_vertices);
-  auto submesh_vertex_index_map = std::make_shared<common::IndexMap>(
-      std::move(submesh_vertex_index_map_pair.first));
-
-  // Create a map from the (local) vertices in the submesh to the
-  // (local) vertices in the mesh
-  std::vector<int32_t> submesh_to_mesh_vertex_map(
-      submesh_owned_vertices.begin(), submesh_owned_vertices.end());
-  submesh_to_mesh_vertex_map.reserve(submesh_vertex_index_map->size_local()
-                                     + submesh_vertex_index_map->num_ghosts());
-  // Add ghost vertices to the map
-  std::transform(submesh_vertex_index_map_pair.second.begin(),
-                 submesh_vertex_index_map_pair.second.end(),
-                 std::back_inserter(submesh_to_mesh_vertex_map),
-                 [size_local = mesh_vertex_index_map->size_local()](
-                     std::int32_t vertex_index)
-                 { return size_local + vertex_index; });
+  const Topology& topology = mesh.topology();
 
   // Get the entities in the submesh that are owned by this process
-  auto mesh_entity_index_map = mesh.topology().index_map(dim);
-  assert(mesh_entity_index_map);
+  auto mesh_map = topology.index_map(dim);
+  assert(mesh_map);
 
-  std::vector<std::int32_t> submesh_owned_entities;
-  std::copy_if(entities.begin(), entities.end(),
-               std::back_inserter(submesh_owned_entities),
-               [size = mesh_entity_index_map->size_local()](std::int32_t e)
-               { return e < size; });
+  std::vector<std::int32_t> submesh_owned;
+  std::copy_if(
+      entities.begin(), entities.end(), std::back_inserter(submesh_owned),
+      [size = mesh_map->size_local()](std::int32_t e) { return e < size; });
+
+  // Create submesh entity index map
+  // TODO Call common::get_owned_indices here? Do we want to
+  // support `entities` possibly having a ghost on one process that is
+  // not in `entities` on the owning process?
+  // TODO: Should entities still be ghosted in the submesh even if they
+  // are not in the `entities` list? If this is not desirable,
+  // create_submap needs to be changed
 
   // Create a map from the (local) entities in the submesh to the
   // (local) entities in the mesh, and create the submesh entity index
   // map.
-  std::vector<int32_t> submesh_to_mesh_entity_map(
-      submesh_owned_entities.begin(), submesh_owned_entities.end());
-  std::shared_ptr<common::IndexMap> submesh_entity_index_map;
-
-  // If the entity dimension is the same as the input mesh topological
-  // dimension, add ghost entities to the submesh. If not, do not add
-  // ghost entities, because in general, not all expected ghost entities
-  // would be present.
-  if (mesh.topology().dim() == dim)
+  std::vector<int32_t> submesh_to_mesh_map(submesh_owned.begin(),
+                                           submesh_owned.end());
+  std::shared_ptr<common::IndexMap> submesh_map;
   {
-    // TODO Call dolfinx::common::get_owned_indices here? Do we want to
-    // support `entities` possibly haveing a ghost on one process that is
-    // not in `entities` on the owning process?
-    std::pair<common::IndexMap, std::vector<int32_t>>
-        submesh_entity_index_map_pair
-        = mesh_entity_index_map->create_submap(submesh_owned_entities);
-    submesh_entity_index_map = std::make_shared<common::IndexMap>(
-        std::move(submesh_entity_index_map_pair.first));
+    std::pair<common::IndexMap, std::vector<int32_t>> map_data
+        = mesh_map->create_submap(submesh_owned);
+    submesh_map = std::make_shared<common::IndexMap>(std::move(map_data.first));
 
     // Add ghost entities to the entity map
-    submesh_to_mesh_entity_map.reserve(
-        submesh_entity_index_map->size_local()
-        + submesh_entity_index_map->num_ghosts());
-    std::transform(submesh_entity_index_map_pair.second.begin(),
-                   submesh_entity_index_map_pair.second.end(),
-                   std::back_inserter(submesh_to_mesh_entity_map),
-                   [size_local = mesh_entity_index_map->size_local()](
-                       std::int32_t entity_index)
-                   { return size_local + entity_index; });
+    submesh_to_mesh_map.reserve(submesh_map->size_local()
+                                + submesh_map->num_ghosts());
+    std::transform(
+        map_data.second.begin(), map_data.second.end(),
+        std::back_inserter(submesh_to_mesh_map),
+        [size_local = mesh_map->size_local()](std::int32_t entity_index)
+        { return size_local + entity_index; });
   }
-  else
+
+  // Get the vertices in the submesh. Use submesh_to_mesh_map
+  // (instead of `entities`) to ensure vertices for ghost entities are
+  // included
+  std::vector<std::int32_t> submesh_vertices
+      = compute_incident_entities(mesh, submesh_to_mesh_map, dim, 0);
+
+  // Get the vertices in the submesh owned by this process
+  auto mesh_index_map0 = topology.index_map(0);
+  assert(mesh_index_map0);
+  std::vector<int32_t> submesh_owned_vertices
+      = common::compute_owned_indices(submesh_vertices, *mesh_index_map0);
+
+  // Create submesh vertex index map
+  std::shared_ptr<common::IndexMap> submesh_map0;
+  std::vector<int32_t> submesh_to_mesh_map0;
   {
-    submesh_entity_index_map = std::make_shared<common::IndexMap>(
-        mesh.comm(), submesh_owned_entities.size());
+    std::pair<common::IndexMap, std::vector<int32_t>> map_data
+        = mesh_index_map0->create_submap(submesh_owned_vertices);
+    submesh_map0
+        = std::make_shared<common::IndexMap>(std::move(map_data.first));
+
+    // Create a map from the (local) vertices in the submesh to the
+    // (local) vertices in the mesh
+    submesh_to_mesh_map0.assign(submesh_owned_vertices.begin(),
+                                submesh_owned_vertices.end());
+    submesh_to_mesh_map0.reserve(submesh_map0->size_local()
+                                 + submesh_map0->num_ghosts());
+
+    // Add ghost vertices to the map
+    std::transform(
+        map_data.second.begin(), map_data.second.end(),
+        std::back_inserter(submesh_to_mesh_map0),
+        [size_local = mesh_index_map0->size_local()](std::int32_t vertex_index)
+        { return size_local + vertex_index; });
   }
 
   // Submesh vertex to vertex connectivity (identity)
   auto submesh_v_to_v = std::make_shared<graph::AdjacencyList<std::int32_t>>(
-      submesh_vertex_index_map->size_local()
-      + submesh_vertex_index_map->num_ghosts());
+      submesh_map0->size_local() + submesh_map0->num_ghosts());
 
   // Submesh entity to vertex connectivity
-  const CellType entity_type
-      = cell_entity_type(mesh.topology().cell_type(), dim, 0);
+  const CellType entity_type = cell_entity_type(topology.cell_type(), dim, 0);
   const int num_vertices_per_entity = cell_num_entities(entity_type, 0);
-  auto mesh_e_to_v = mesh.topology().connectivity(dim, 0);
+  auto mesh_e_to_v = topology.connectivity(dim, 0);
   std::vector<std::int32_t> submesh_e_to_v_vec;
-  submesh_e_to_v_vec.reserve(submesh_to_mesh_entity_map.size()
+  submesh_e_to_v_vec.reserve(submesh_to_mesh_map.size()
                              * num_vertices_per_entity);
   std::vector<std::int32_t> submesh_e_to_v_offsets(1, 0);
-  submesh_e_to_v_offsets.reserve(submesh_to_mesh_entity_map.size() + 1);
-  for (std::int32_t e : submesh_to_mesh_entity_map)
+  submesh_e_to_v_offsets.reserve(submesh_to_mesh_map.size() + 1);
+
+  // Create mesh to submesh vertex map (i.e. the inverse of
+  // submesh_to_mesh_map0)
+  // NOTE: Depending on the submesh, this may be densely or sparsely
+  // populated. Is a different data structure more appropriate?
+  std::vector<std::int32_t> mesh_to_submesh_map0(
+      mesh_index_map0->size_local() + mesh_index_map0->num_ghosts(), -1);
+  for (std::size_t i = 0; i < submesh_to_mesh_map0.size(); ++i)
+    mesh_to_submesh_map0[submesh_to_mesh_map0[i]] = i;
+
+  for (std::int32_t e : submesh_to_mesh_map)
   {
     std::span<const std::int32_t> vertices = mesh_e_to_v->links(e);
     for (std::int32_t v : vertices)
     {
-      auto it = std::find(submesh_to_mesh_vertex_map.begin(),
-                          submesh_to_mesh_vertex_map.end(), v);
-      assert(it != submesh_to_mesh_vertex_map.end());
-      submesh_e_to_v_vec.push_back(
-          std::distance(submesh_to_mesh_vertex_map.begin(), it));
+      std::int32_t v_submesh = mesh_to_submesh_map0[v];
+      assert(v_submesh != -1);
+      submesh_e_to_v_vec.push_back(v_submesh);
     }
     submesh_e_to_v_offsets.push_back(submesh_e_to_v_vec.size());
   }
@@ -324,30 +308,71 @@ mesh::create_submesh(const Mesh& mesh, int dim,
 
   // Create submesh topology
   Topology submesh_topology(mesh.comm(), entity_type);
-  submesh_topology.set_index_map(0, submesh_vertex_index_map);
-  submesh_topology.set_index_map(dim, submesh_entity_index_map);
+  submesh_topology.set_index_map(0, submesh_map0);
+  submesh_topology.set_index_map(dim, submesh_map);
   submesh_topology.set_connectivity(submesh_v_to_v, 0, 0);
   submesh_topology.set_connectivity(submesh_e_to_v, dim, 0);
 
   // -- Submesh geometry
+  const Geometry& geometry = mesh.geometry();
 
   // Get the geometry dofs in the submesh based on the entities in
   // submesh
-  const std::vector<std::int32_t> e_to_g
-      = entities_to_geometry(mesh, dim, submesh_to_mesh_entity_map, false);
-  const std::size_t num_vertices = mesh::num_cell_vertices(
-      cell_entity_type(mesh.topology().cell_type(), dim, 0));
+  const fem::ElementDofLayout layout = geometry.cmap().create_dof_layout();
+  // NOTE: Unclear what this return for prisms
+  const std::size_t num_entity_dofs = layout.num_entity_closure_dofs(dim);
 
-  std::vector<int32_t> submesh_x_dofs = e_to_g;
+  std::vector<std::int32_t> geometry_indices;
+  geometry_indices.reserve(num_entity_dofs * submesh_to_mesh_map.size());
+  std::vector<std::int32_t> submesh_x_dofmap_offsets;
+  submesh_x_dofmap_offsets.reserve(submesh_to_mesh_map.size() + 1);
+  submesh_x_dofmap_offsets.push_back(0);
+  {
+    const graph::AdjacencyList<std::int32_t>& xdofs = geometry.dofmap();
+    const int tdim = topology.dim();
+    mesh.topology_mutable().create_entities(dim);
+    mesh.topology_mutable().create_connectivity(dim, tdim);
+    mesh.topology_mutable().create_connectivity(tdim, dim);
+
+    // Fetch connectivities required to get entity dofs
+    const std::vector<std::vector<std::vector<int>>>& closure_dofs
+        = layout.entity_closure_dofs_all();
+    std::shared_ptr<const graph::AdjacencyList<int>> e_to_c
+        = topology.connectivity(dim, tdim);
+    assert(e_to_c);
+    std::shared_ptr<const graph::AdjacencyList<int>> c_to_e
+        = topology.connectivity(tdim, dim);
+    assert(c_to_e);
+    for (std::size_t i = 0; i < submesh_to_mesh_map.size(); ++i)
+    {
+      const std::int32_t idx = submesh_to_mesh_map[i];
+      assert(!e_to_c->links(idx).empty());
+      // Always pick the last cell to be consistent with the e_to_v connectivity
+      const std::int32_t cell = e_to_c->links(idx).back();
+      auto cell_entities = c_to_e->links(cell);
+      auto it = std::find(cell_entities.begin(), cell_entities.end(), idx);
+      assert(it != cell_entities.end());
+      const auto local_entity = std::distance(cell_entities.begin(), it);
+      const std::vector<std::int32_t>& entity_dofs
+          = closure_dofs[dim][local_entity];
+
+      auto xc = xdofs.links(cell);
+      for (std::int32_t entity_dof : entity_dofs)
+        geometry_indices.push_back(xc[entity_dof]);
+      submesh_x_dofmap_offsets.push_back(geometry_indices.size());
+    }
+  }
+
+  std::vector<std::int32_t> submesh_x_dofs = geometry_indices;
   std::sort(submesh_x_dofs.begin(), submesh_x_dofs.end());
   submesh_x_dofs.erase(
       std::unique(submesh_x_dofs.begin(), submesh_x_dofs.end()),
       submesh_x_dofs.end());
 
   // Get the geometry dofs in the submesh owned by this process
-  auto mesh_geometry_dof_index_map = mesh.geometry().index_map();
+  auto mesh_geometry_dof_index_map = geometry.index_map();
   assert(mesh_geometry_dof_index_map);
-  auto submesh_owned_x_dofs = dolfinx::common::compute_owned_indices(
+  auto submesh_owned_x_dofs = common::compute_owned_indices(
       submesh_x_dofs, *mesh_geometry_dof_index_map);
 
   // Create submesh geometry index map
@@ -379,48 +404,43 @@ mesh::create_submesh(const Mesh& mesh, int dim,
   std::vector<double> submesh_x(3 * submesh_num_x_dofs);
   for (int i = 0; i < submesh_num_x_dofs; ++i)
   {
-    common::impl::copy_N<3>(
-        std::next(mesh_x.begin(), 3 * submesh_to_mesh_x_dof_map[i]),
-        std::next(submesh_x.begin(), 3 * i));
+    std::copy_n(std::next(mesh_x.begin(), 3 * submesh_to_mesh_x_dof_map[i]), 3,
+                std::next(submesh_x.begin(), 3 * i));
   }
 
-  std::vector<std::int32_t> entity_x_dofs;
+  // Create mesh to submesh geometry map
+  std::vector<std::int32_t> mesh_to_submesh_x_dof_map(
+      mesh_geometry_dof_index_map->size_local()
+          + mesh_geometry_dof_index_map->num_ghosts(),
+      -1);
+  for (std::size_t i = 0; i < submesh_to_mesh_x_dof_map.size(); ++i)
+    mesh_to_submesh_x_dof_map[submesh_to_mesh_x_dof_map[i]] = i;
 
-  // Crete submesh geometry dofmap
+  // Create submesh geometry dofmap
   std::vector<std::int32_t> submesh_x_dofmap_vec;
-  submesh_x_dofmap_vec.reserve(e_to_g.size());
-  std::vector<std::int32_t> submesh_x_dofmap_offsets(1, 0);
-  submesh_x_dofmap_offsets.reserve(submesh_to_mesh_entity_map.size() + 1);
-  for (std::size_t i = 0; i < submesh_to_mesh_entity_map.size(); ++i)
-  {
-    // Get the mesh geometry dofs for ith entity in entities
-    auto it = std::next(e_to_g.begin(), i * num_vertices);
-    entity_x_dofs.assign(it, std::next(it, num_vertices));
+  submesh_x_dofmap_vec.reserve(geometry_indices.size());
+  std::transform(geometry_indices.cbegin(), geometry_indices.cend(),
+                 std::back_inserter(submesh_x_dofmap_vec),
+                 [&mesh_to_submesh_x_dof_map](auto x_dof)
+                 {
+                   std::int32_t x_dof_submesh
+                       = mesh_to_submesh_x_dof_map[x_dof];
+                   assert(x_dof_submesh != -1);
+                   return x_dof_submesh;
+                 });
 
-    // For each mesh dof of the entity, get the submesh dof
-    for (std::int32_t x_dof : entity_x_dofs)
-    {
-      auto it = std::find(submesh_to_mesh_x_dof_map.begin(),
-                          submesh_to_mesh_x_dof_map.end(), x_dof);
-      assert(it != submesh_to_mesh_x_dof_map.end());
-      submesh_x_dofmap_vec.push_back(
-          std::distance(submesh_to_mesh_x_dof_map.begin(), it));
-    }
-    submesh_x_dofmap_offsets.push_back(submesh_x_dofmap_vec.size());
-  }
   graph::AdjacencyList<std::int32_t> submesh_x_dofmap(
       std::move(submesh_x_dofmap_vec), std::move(submesh_x_dofmap_offsets));
 
   // Create submesh coordinate element
   CellType submesh_coord_cell
-      = cell_entity_type(mesh.geometry().cmap().cell_shape(), dim, 0);
-  auto submesh_coord_ele = fem::CoordinateElement(
-      submesh_coord_cell, mesh.geometry().cmap().degree());
+      = cell_entity_type(geometry.cmap().cell_shape(), dim, 0);
+  fem::CoordinateElement submesh_coord_ele(
+      submesh_coord_cell, geometry.cmap().degree(), geometry.cmap().variant());
 
   // Submesh geometry input_global_indices
   // TODO Check this
-  const std::vector<std::int64_t>& mesh_igi
-      = mesh.geometry().input_global_indices();
+  const std::vector<std::int64_t>& mesh_igi = geometry.input_global_indices();
   std::vector<std::int64_t> submesh_igi;
   submesh_igi.reserve(submesh_to_mesh_x_dof_map.size());
   std::transform(submesh_to_mesh_x_dof_map.begin(),
@@ -432,12 +452,11 @@ mesh::create_submesh(const Mesh& mesh, int dim,
   // Create geometry
   Geometry submesh_geometry(
       submesh_x_dof_index_map, std::move(submesh_x_dofmap), submesh_coord_ele,
-      std::move(submesh_x), mesh.geometry().dim(), std::move(submesh_igi));
+      std::move(submesh_x), geometry.dim(), std::move(submesh_igi));
 
   return {Mesh(mesh.comm(), std::move(submesh_topology),
                std::move(submesh_geometry)),
-          std::move(submesh_to_mesh_entity_map),
-          std::move(submesh_to_mesh_vertex_map),
+          std::move(submesh_to_mesh_map), std::move(submesh_to_mesh_map0),
           std::move(submesh_to_mesh_x_dof_map)};
 }
 //-----------------------------------------------------------------------------
