@@ -624,7 +624,7 @@ plaza::refine(const mesh::Mesh& mesh, bool redistribute,
             std::move(parent_cell), std::move(parent_facet)};
   }
 
-  const std::shared_ptr<const common::IndexMap> map_c
+  std::shared_ptr<const common::IndexMap> map_c
       = mesh.topology().index_map(mesh.topology().dim());
   const int num_ghost_cells = map_c->num_ghosts();
   // Check if mesh has ghost cells on any rank
@@ -644,9 +644,8 @@ plaza::refine(const mesh::Mesh& mesh, bool redistribute,
 }
 //-----------------------------------------------------------------------------
 std::tuple<mesh::Mesh, std::vector<std::int32_t>, std::vector<std::int8_t>>
-plaza::refine(const mesh::Mesh& mesh,
-              const std::span<const std::int32_t>& edges, bool redistribute,
-              RefinementOptions options)
+plaza::refine(const mesh::Mesh& mesh, std::span<const std::int32_t> edges,
+              bool redistribute, RefinementOptions options)
 {
 
   auto [cell_adj, new_vertex_coords, xshape, parent_cell, parent_facet]
@@ -659,7 +658,7 @@ plaza::refine(const mesh::Mesh& mesh,
             std::move(parent_cell), std::move(parent_facet)};
   }
 
-  const std::shared_ptr<const common::IndexMap> map_c
+  std::shared_ptr<const common::IndexMap> map_c
       = mesh.topology().index_map(mesh.topology().dim());
   const int num_ghost_cells = map_c->num_ghosts();
   // Check if mesh has ghost cells on any rank
@@ -737,7 +736,7 @@ std::tuple<graph::AdjacencyList<std::int64_t>, std::vector<double>,
            std::array<std::size_t, 2>, std::vector<std::int32_t>,
            std::vector<std::int8_t>>
 plaza::compute_refinement_data(const mesh::Mesh& mesh,
-                               const std::span<const std::int32_t>& edges,
+                               std::span<const std::int32_t> edges,
                                RefinementOptions options)
 {
   if (mesh.topology().cell_type() != mesh::CellType::triangle
