@@ -20,9 +20,10 @@ namespace
 
 void test_fides_mesh()
 {
-  auto mesh = std::make_shared<mesh::Mesh>(mesh::create_rectangle(
-      MPI_COMM_WORLD, {{{0.0, 0.0}, {1.0, 1.0}}}, {22, 12},
-      mesh::CellType::triangle, mesh::GhostMode::shared_facet));
+  auto part = mesh::create_cell_partitioner(mesh::GhostMode::shared_facet);
+  auto mesh = std::make_shared<mesh::Mesh>(
+      mesh::create_rectangle(MPI_COMM_WORLD, {{{0.0, 0.0}, {1.0, 1.0}}},
+                             {22, 12}, mesh::CellType::triangle, part));
   io::FidesWriter writer(mesh->comm(), "test_mesh.bp", mesh);
   writer.write(0.0);
 
