@@ -43,7 +43,7 @@ class FormMetaClass:
             coeffs: Finite element coefficients that appear in the form
             constants: Constants appearing in the form
             subdomains: Subdomains for integrals
-            mesh: The mesh that the form is deined on
+            mesh: The mesh that the form is defined on
 
         """
         self._code = code
@@ -87,14 +87,14 @@ form_types = typing.Union[FormMetaClass, _cpp.fem.Form_float32, _cpp.fem.Form_fl
 
 
 def form(form: typing.Union[ufl.Form, typing.Iterable[ufl.Form]], dtype: np.dtype = PETSc.ScalarType,
-         form_compiler_params: dict = {}, jit_params: dict = {}):
+         form_compiler_options: dict = {}, jit_options: dict = {}):
     """Create a DOLFINx Form or an array of Forms
 
     Args:
         form: A UFL form or list(s) of UFL forms
         dtype: Scalar type to use for the compiled form
-        form_compiler_params: See :func:`ffcx_jit <dolfinx.jit.ffcx_jit>`
-        jit_params:See :func:`ffcx_jit <dolfinx.jit.ffcx_jit>`
+        form_compiler_options: See :func:`ffcx_jit <dolfinx.jit.ffcx_jit>`
+        jit_options:See :func:`ffcx_jit <dolfinx.jit.ffcx_jit>`
 
     Returns:
         Compiled finite element Form
@@ -110,13 +110,16 @@ def form(form: typing.Union[ufl.Form, typing.Iterable[ufl.Form]], dtype: np.dtyp
     """
     if dtype == np.float32:
         ftype = _cpp.fem.Form_float32
-        form_compiler_params["scalar_type"] = "float"
+        form_compiler_options["scalar_type"] = "float"
     elif dtype == np.float64:
         ftype = _cpp.fem.Form_float64
-        form_compiler_params["scalar_type"] = "double"
+        form_compiler_options["scalar_type"] = "double"
+    elif dtype == np.complex64:
+        ftype = _cpp.fem.Form_complex64
+        form_compiler_options["scalar_type"] = "float _Complex"
     elif dtype == np.complex128:
         ftype = _cpp.fem.Form_complex128
-        form_compiler_params["scalar_type"] = "double _Complex"
+        form_compiler_options["scalar_type"] = "double _Complex"
     else:
         raise NotImplementedError(f"Type {dtype} not supported.")
 
@@ -126,15 +129,22 @@ def form(form: typing.Union[ufl.Form, typing.Iterable[ufl.Form]], dtype: np.dtyp
         """"Compile a single UFL form"""
         # Extract subdomain data from UFL form
         sd = form.subdomain_data()
-        subdomains, = list(sd.values())  # Assuming single domain
         domain, = list(sd.keys())  # Assuming single domain
+        # Get subdomain data for each integral type
+        subdomains = {}
+        for integral_type, data in sd.get(domain).items():
+            # Check that the subdomain data for each integral of this type is
+            # the same
+            assert all([id(d) == id(data[0]) for d in data])
+            subdomains[integral_type] = data[0]
+
         mesh = domain.ufl_cargo()
         if mesh is None:
             raise RuntimeError("Expecting to find a Mesh in the form.")
 
         ufcx_form, module, code = jit.ffcx_jit(mesh.comm, form,
-                                               form_compiler_params=form_compiler_params,
-                                               jit_params=jit_params)
+                                               form_compiler_options=form_compiler_options,
+                                               jit_options=jit_options)
 
         # For each argument in form extract its function space
         V = [arg.ufl_function_space()._cpp_object for arg in form.arguments()]
