@@ -14,6 +14,7 @@ from __future__ import annotations
 
 import contextlib
 import functools
+import os
 import typing
 
 import ufl
@@ -30,6 +31,8 @@ from dolfinx.fem.forms import form as _create_form
 from dolfinx.fem.forms import form_types
 from dolfinx.fem.function import Function as _Function
 
+import petsc4py
+import petsc4py.lib
 from petsc4py import PETSc
 
 
@@ -533,7 +536,7 @@ class LinearProblem:
     """
 
     def __init__(self, a: ufl.Form, L: ufl.Form, bcs: typing.List[DirichletBCMetaClass] = [],
-                 u: _Function = None, petsc_options={}, form_compiler_params={}, jit_params={}):
+                 u: typing.Optional[_Function] = None, petsc_options={}, form_compiler_options={}, jit_options={}):
         """Initialize solver for a linear variational problem.
 
         Args:
@@ -541,27 +544,29 @@ class LinearProblem:
             L: A linear UFL form, the right hand side of the variational problem.
             bcs: A list of Dirichlet boundary conditions.
             u: The solution function. It will be created if not provided.
-            petsc_options: Parameters that is passed to the linear
+            petsc_options: Options that are passed to the linear
                 algebra backend PETSc. For available choices for the
                 'petsc_options' kwarg, see the `PETSc documentation
                 <https://petsc4py.readthedocs.io/en/stable/manual/ksp/>`_.
-            form_compiler_params: Parameters used in FFCx compilation of
+            form_compiler_options: Options used in FFCx compilation of
                 this form. Run ``ffcx --help`` at the commandline to see
                 all available options.
-            jit_params: Parameters used in CFFI JIT compilation of C
+            jit_options: Options used in CFFI JIT compilation of C
                 code generated by FFCx. See `python/dolfinx/jit.py` for
-                all available parameters. Takes priority over all other
-                parameter values.
+                all available options. Takes priority over all other
+                option values.
 
         Example::
 
-            problem = LinearProblem(a, L, [bc0, bc1], petsc_options={"ksp_type": "preonly", "pc_type": "lu"})
-
+            problem = LinearProblem(a, L, [bc0, bc1],
+                                    petsc_options={"ksp_type": "preonly",
+                                                   "pc_type": "lu",
+                                                   "pc_factor_mat_solver_type": "mumps"})
         """
-        self._a = _create_form(a, form_compiler_params=form_compiler_params, jit_params=jit_params)
+        self._a = _create_form(a, form_compiler_options=form_compiler_options, jit_options=jit_options)
         self._A = create_matrix(self._a)
 
-        self._L = _create_form(L, form_compiler_params=form_compiler_params, jit_params=jit_params)
+        self._L = _create_form(L, form_compiler_options=form_compiler_options, jit_options=jit_options)
         self._b = create_vector(self._L)
 
         if u is None:
@@ -654,7 +659,7 @@ class NonlinearProblem:
     """
 
     def __init__(self, F: ufl.form.Form, u: _Function, bcs: typing.List[DirichletBCMetaClass] = [],
-                 J: ufl.form.Form = None, form_compiler_params={}, jit_params={}):
+                 J: ufl.form.Form = None, form_compiler_options={}, jit_options={}):
         """Initialize solver for solving a non-linear problem using Newton's method, :math:`dF/du(u) du = -F(u)`.
 
         Args:
@@ -662,21 +667,21 @@ class NonlinearProblem:
             u: The unknown
             bcs: List of Dirichlet boundary conditions
             J: UFL representation of the Jacobian (Optional)
-            form_compiler_params: Parameters used in FFCx
+            form_compiler_options: Options used in FFCx
                 compilation of this form. Run ``ffcx --help`` at the
                 commandline to see all available options.
-            jit_params: Parameters used in CFFI JIT compilation of C
+            jit_options: Options used in CFFI JIT compilation of C
                 code generated by FFCx. See ``python/dolfinx/jit.py``
-                for all available parameters. Takes priority over all
-                other parameter values.
+                for all available options. Takes priority over all
+                other option values.
 
         Example::
 
             problem = LinearProblem(F, u, [bc0, bc1])
 
         """
-        self._L = _create_form(F, form_compiler_params=form_compiler_params,
-                               jit_params=jit_params)
+        self._L = _create_form(F, form_compiler_options=form_compiler_options,
+                               jit_options=jit_options)
 
         # Create the Jacobian matrix, dF/du
         if J is None:
@@ -684,8 +689,8 @@ class NonlinearProblem:
             du = ufl.TrialFunction(V)
             J = ufl.derivative(F, u, du)
 
-        self._a = _create_form(J, form_compiler_params=form_compiler_params,
-                               jit_params=jit_params)
+        self._a = _create_form(J, form_compiler_options=form_compiler_options,
+                               jit_options=jit_options)
         self.bcs = bcs
 
     @property
@@ -736,3 +741,42 @@ class NonlinearProblem:
         A.zeroEntries()
         _assemble_matrix_mat(A, self._a, self.bcs)
         A.assemble()
+
+
+def load_petsc_lib(loader: typing.Callable[[str], typing.Any]) -> typing.Any:
+    """
+    Load PETSc shared library using loader callable, e.g. ctypes.CDLL.
+
+    Args:
+        loader: A callable that accepts a library path and returns a wrapped library.
+
+    Returns:
+        A wrapped library of the type returned by the callable.
+    """
+    petsc_lib = None
+
+    petsc_dir = petsc4py.get_config()['PETSC_DIR']
+    petsc_arch = petsc4py.lib.getPathArchPETSc()[1]
+
+    candidate_paths = [os.path.join(petsc_dir, petsc_arch, "lib", "libpetsc.so"),
+                       os.path.join(petsc_dir, petsc_arch, "lib", "libpetsc.dylib")]
+
+    exists_paths = []
+    for candidate_path in candidate_paths:
+        if os.path.exists(candidate_path):
+            exists_paths.append(candidate_path)
+
+    if len(exists_paths) == 0:
+        raise RuntimeError("Could not find a PETSc shared library.")
+
+    for exists_path in exists_paths:
+        try:
+            petsc_lib = loader(exists_path)
+        except OSError:
+            print(f"Failed to load shared library found at {exists_path}.")
+            continue
+
+    if petsc_lib is None:
+        raise RuntimeError("Failed to load a PETSc shared library.")
+
+    return petsc_lib
