@@ -308,8 +308,6 @@ def test_assembly_solve_block_nl():
 
         snes = PETSc.SNES().create(MPI.COMM_WORLD)
         snes.setTolerances(rtol=1.0e-15, max_it=10)
-        snes.getKSP().setType("preonly")
-        snes.getKSP().getPC().setType("lu")
 
         problem = NonlinearPDE_SNESProblem(F, J, [u, p], bcs)
         snes.setFunction(problem.F_block, Fvec)
@@ -346,12 +344,6 @@ def test_assembly_solve_block_nl():
         snes.getKSP().getPC().setType("fieldsplit")
         snes.getKSP().getPC().setFieldSplitIS(["u", nested_IS[0][0]], ["p", nested_IS[1][1]])
 
-        ksp_u, ksp_p = snes.getKSP().getPC().getFieldSplitSubKSP()
-        ksp_u.setType("preonly")
-        ksp_u.getPC().setType('lu')
-        ksp_p.setType("preonly")
-        ksp_p.getPC().setType('lu')
-
         problem = NonlinearPDE_SNESProblem(F, J, [u, p], bcs)
         snes.setFunction(problem.F_nest, Fvec)
         snes.setJacobian(problem.J_nest, J=Jmat, P=None)
@@ -402,9 +394,6 @@ def test_assembly_solve_block_nl():
         snes = PETSc.SNES().create(MPI.COMM_WORLD)
         snes.setTolerances(rtol=1.0e-15, max_it=10)
 
-        snes.getKSP().setType("preonly")
-        snes.getKSP().getPC().setType("lu")
-
         problem = NonlinearPDE_SNESProblem(F, J, U, bcs)
         snes.setFunction(problem.F_mono, Fvec)
         snes.setJacobian(problem.J_mono, J=Jmat, P=None)
@@ -494,7 +483,6 @@ def test_assembly_solve_taylor_hood_nl(mesh):
     snes = PETSc.SNES().create(MPI.COMM_WORLD)
     snes.setTolerances(rtol=1.0e-15, max_it=10)
     snes.getKSP().setType("minres")
-    snes.getKSP().getPC().setType("lu")
 
     problem = NonlinearPDE_SNESProblem(F, J, [u, p], bcs, P=P)
     snes.setFunction(problem.F_block, Fvec0)
@@ -530,12 +518,6 @@ def test_assembly_solve_taylor_hood_nl(mesh):
     snes.getKSP().getPC().setType("fieldsplit")
     snes.getKSP().getPC().setFieldSplitIS(["u", nested_IS[0][0]], ["p", nested_IS[1][1]])
 
-    ksp_u, ksp_p = snes.getKSP().getPC().getFieldSplitSubKSP()
-    ksp_u.setType("preonly")
-    ksp_u.getPC().setType('lu')
-    ksp_p.setType("preonly")
-    ksp_p.getPC().setType('lu')
-
     problem = NonlinearPDE_SNESProblem(F, J, [u, p], bcs, P=P)
     snes.setFunction(problem.F_nest, Fvec1)
     snes.setJacobian(problem.J_nest, J=Jmat1, P=Pmat1)
@@ -589,7 +571,6 @@ def test_assembly_solve_taylor_hood_nl(mesh):
     snes = PETSc.SNES().create(MPI.COMM_WORLD)
     snes.setTolerances(rtol=1.0e-15, max_it=10)
     snes.getKSP().setType("minres")
-    snes.getKSP().getPC().setType("lu")
 
     problem = NonlinearPDE_SNESProblem(F, J, U, bcs, P=P)
     snes.setFunction(problem.F_mono, Fvec2)
