@@ -12,9 +12,9 @@ import pytest
 import ufl
 from dolfinx.common import has_adios2
 from dolfinx.fem import Function, FunctionSpace, VectorFunctionSpace
-from dolfinx.mesh import (CellType, create_mesh, create_unit_cube,
-                          create_unit_square, GhostMode)
 from dolfinx.graph import create_adjacencylist
+from dolfinx.mesh import (CellType, create_mesh, create_unit_cube,
+                          create_unit_square)
 
 from mpi4py import MPI
 
@@ -24,13 +24,13 @@ except ImportError:
     pytest.skip("Test require ADIOS2", allow_module_level=True)
 
 
-def generate_mesh(dim: int, simplex: bool, N: int = 3):
+def generate_mesh(dim: int, simplex: bool, N: int = 5):
     """Helper function for parametrizing over meshes"""
     if dim == 2:
         if simplex:
             return create_unit_square(MPI.COMM_WORLD, N, N)
         else:
-            return create_unit_square(MPI.COMM_WORLD, N, N, CellType.quadrilateral)
+            return create_unit_square(MPI.COMM_WORLD, 2 * N, N, CellType.quadrilateral)
     elif dim == 3:
         if simplex:
             return create_unit_cube(MPI.COMM_WORLD, N, N, N)
@@ -107,9 +107,9 @@ def test_fides_function_at_nodes(tempdir, dim, simplex):
 
             mesh.geometry.x[:, :2] += 0.1
             if mesh.geometry.dim == 2:
-                v.interpolate(lambda x: (t * x[0], x[1] + x[1] * 1j))
+                v.interpolate(lambda x: np.vstack((t * x[0], x[1] + x[1] * 1j)))
             elif mesh.geometry.dim == 3:
-                v.interpolate(lambda x: (t * x[2], x[0] + x[2] * 2j, x[1]))
+                v.interpolate(lambda x: np.vstack((t * x[2], x[0] + x[2] * 2j, x[1])))
             f.write(t)
 
 
@@ -184,27 +184,28 @@ def test_vtx_single_function(tempdir, dim, simplex):
 
 
 @pytest.mark.skipif(not has_adios2, reason="Requires ADIOS2.")
+@pytest.mark.parametrize("dtype", [np.float32, np.float64, np.complex64, np.complex128])
 @pytest.mark.parametrize("dim", [2, 3])
 @pytest.mark.parametrize("simplex", [True, False])
-def test_vtx_functions(tempdir, dim, simplex):
+def test_vtx_functions(tempdir, dtype, dim, simplex):
     "Test saving high order Lagrange functions"
     mesh = generate_mesh(dim, simplex)
     V = VectorFunctionSpace(mesh, ("DG", 2))
-    v = Function(V)
+    v = Function(V, dtype=dtype)
     bs = V.dofmap.index_map_bs
 
     def vel(x):
-        values = np.zeros((dim, x.shape[1]))
+        values = np.zeros((dim, x.shape[1]), dtype=dtype)
         values[0] = x[1]
         values[1] = x[0]
         return values
     v.interpolate(vel)
 
     W = FunctionSpace(mesh, ("DG", 2))
-    w = Function(W)
+    w = Function(W, dtype=v.dtype)
     w.interpolate(lambda x: x[0] + x[1])
 
-    filename = Path(tempdir, "v.bp")
+    filename = Path(tempdir, f"v-{np.dtype(dtype).num}.bp")
     f = VTXWriter(mesh.comm, filename, [v, w])
 
     # Set two cells to 0
@@ -248,7 +249,7 @@ def test_empty_rank_mesh(tempdir):
     domain = ufl.Mesh(
         ufl.VectorElement("Lagrange", ufl.Cell(cell_type.name), 1))
 
-    def partitioner(comm, nparts, local_graph, num_ghost_nodes, ghosting):
+    def partitioner(comm, nparts, local_graph, num_ghost_nodes):
         """Leave cells on the current rank"""
         dest = np.full(len(cells), comm.rank, dtype=np.int32)
         return create_adjacencylist(dest)
@@ -261,7 +262,7 @@ def test_empty_rank_mesh(tempdir):
         cells = create_adjacencylist(np.empty((0, 3), dtype=np.int64))
         x = np.empty((0, 2), dtype=np.float64)
 
-    mesh = create_mesh(comm, cells, x, domain, GhostMode.none, partitioner)
+    mesh = create_mesh(comm, cells, x, domain, partitioner)
 
     V = FunctionSpace(mesh, ("Lagrange", 1))
     u = Function(V)
